/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	February 03, 2009


#pragma once

#ifdef WIN32
	#include <windows.h>
	#include <GL/gl.h>
	#include <GL/glu.h>
	#include <GL/glext.h>

	#ifdef min
	#undef min
	#endif

	#ifdef max
	#undef max
	#endif
#elif defined(linux)
	#include <GL/gl.h>
	#include <GL/glu.h>
	#include <GL/glext.h>
	#include <GL/glx.h>
	#include <GL/glxext.h>
#elif defined(__APPLE__)
	#include <OpenGL/gl.h>
	#include <OpenGL/glu.h>
	#include <OpenGL/glext.h>
#endif

#include <vector>
#include <string>
#include <cstring>
#include <algorithm>

#if defined(WIN32)
#define wglXGetProcAddress wglGetProcAddress
#else
#define wglXGetProcAddress(x) glXGetProcAddress((GLubyte*) x)
#endif

/**
 *
 * @author Jakob Bieling
 */
std::vector <char> get_extensions ();
namespace extensions
{
	static bool is_supported (std::string ext)
	{
		static std::vector <char> exts = get_extensions ();

		std::transform (ext.begin(), ext.end(), ext.begin(), toupper);

		ext.insert (0, 1, ' ');
		ext.append (1, ' ');

		return strstr (&exts.front (), ext.c_str ()) > 0;
	}
};
