/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	April 17, 2009

#pragma once

/**	Use a shader to fill the last portion of the A-matrix
 *	appropriatly. Otherwise, that part will first be zeroed
 *	and then all necessary elements will be set to -1 one
 *	by one.
 */
#define USE_ART_SHADER



/**	Use one-channel textures as defined by nVidia. This type
 *	of texture is most probably not available on non-nVidia
 *	hardware. Otherwise, RGBA textures will be used where
 *	only the red channel is in use.
 */
#define USE_1CHN_TEXTURE



/**	Use pixel-buffer objects (PBOs) to transfer data from
 *	main memory to texture memory. Unfortunately, using a
 *	regular RGBA format results in having to transfer all
 *	channels, ie. calling glTexSubImage with GL_RED only
 *	will fail. This sort of defeats the purpose of PBOs,
 *	which is why they will only be used when the nVidia
 *	texture format is enabled as well. If not defined, the
 *	regular glTexSubImage call will be used.
 */
#define USE_PIXEL_BUFFER
