/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	May 4, 2011


#pragma once

#include "matrix.h"
#include "engine.h"

#include "../lp_problem.h"
#include "../lp_result.h"

#include <cmath>

enum pricing_e
{
	dantzig,
	steepest_edge
};

template <typename env>
class revised_simplex_impl;
/*
class revised_simplex_cpu
{
	static revised_simplex_impl <cpu::environment>*	impl;

public:

	revised_simplex_cpu ();

	void			precache ();

	void			set_constraints (int rows, int cols, double const fa [], double const fb [], double const fc [],
									 double const* fu = 0, double const* fl = 0, bounds_e const* bb = 0);

	feasability_e	is_feasible_simple ();
	feasability_e	is_feasible_regular ();
	feasability_e	is_feasible_bounded ();

	feasability_e	solve_optimal_simple ();
	feasability_e	solve_optimal_regular ();
	feasability_e	solve_optimal_bounded ();

	int				get_result_vector (float* fx, int sz) const;
	float			get_objective_value () const;
	int				get_num_iterations () const;

	pricing_e		pricing;
};

*/
class revised_simplex_gpu
{
	static revised_simplex_impl <gpu::environment>*	impl;
public:

	revised_simplex_gpu ();

	void			precache ();

	/**
	 * Set the constraint matrix A and vector b of the system Ax <= b
	 *
	 * @note This function also allocates all resources to be able to store
	 *		 all date of the LP.
	 *
	 * @param rows	Number of rows in the matrix A stored in 'fa'. It also serves
	 *				as the number of elements in the vector b stored in 'fb'.
	 * @param cols	Number of columns in the matrix A stored in 'fa'.
	 * @param fa	Array holding the matrix A in row-major order.
	 * @param fb	Array holding the vector b.
	 * @param fc	Array holding the vector of objective values. If not specified,
	 *				the objective values of all x are set to 0, effectively testing
	 *				for feasibility only.
	 * @param fl	Array holding the vector of lower bounds. If not specified, all
	 *				lower bounds will be set to 0.
	 * @param fu	Array holding the vector of upper bounds. If not specified, no
	 *				upper bounds will be used.
	 * @param bb	Array of values specifying which bounds apply for each
	 *				element. Each element can be either 'no_bounds', 'upper_bound',
	 *				'lower_bound', or 'double_bound'. The values only have a meaning
	 *				if their respective arrays (fu or fl) have been specified; i.e.,
	 *				passing fu=0 and using 'bound_double' effectively means 'lower_bound'
	 *				only!
	*/
	bool set_constraints (std::vector <double> const& vA,
						  std::vector <double> const& vb,
						  std::vector <double> const& vc,
						  double const* fub = 0, double const* flb = 0);

	feasability_e	is_feasible_simple ();
	feasability_e	is_feasible_regular ();
	feasability_e	is_feasible_bounded ();

	feasability_e	solve_optimal_simple ();
	feasability_e	solve_optimal_regular ();
	feasability_e	solve_optimal_bounded ();

	int				get_result_vector (float* fx, int sz) const;
	float			get_objective_value () const;
	int				get_num_iterations () const;

	pricing_e		pricing;
	void (*progress_cb) ();
};

