/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	February 03, 2009


#pragma once

#include "arg_parser.h"
#include "lp_result.h"
#include <vector>
#include <cstdlib>


template <size_t C>
void log_benchmark (char const* logname, int problem_size, int problem_seed, lp_result_t (&res) [C])
{
	char statusIDs [C * 2] = {};
	for (size_t i = 0; i < C; ++ i)
	{
		statusIDs [i * 2] = '?';
		statusIDs [i * 2 + 1] = ' ';
		
		switch (res [i].status)
		{
			case undefined:		statusIDs [i * 2] = '?'; break;
			case aborted:		statusIDs [i * 2] = 'a'; break;
			case infeasible:	statusIDs [i * 2] = 'i'; break;
			case feasible:		statusIDs [i * 2] = 'f'; break;
			case unbounded:		statusIDs [i * 2] = 'u'; break;
			case optimal:		statusIDs [i * 2] = 'o'; break;
			default:			statusIDs [i * 2] = '-'; break;
		}
	}
	statusIDs [C * 2 - 1] = 0;

	char const gpu_fail = (res [imp_gpu].status == undefined || res [imp_glpk].status == undefined || res [imp_gpu] == res [imp_glpk]) ? ' ' : '!';
	
	char tmp [1024];
	int len = 0;

	len += sprintf (tmp + len, "% 4i % 3i ", problem_size, problem_seed);
	for (size_t i = 0; i < C; ++ i)
		len += sprintf (tmp + len, "% 3i ", res [i].it_cnt);
	if (!SAT_ONLY)
	{
		for (size_t i = 0; i < C; ++ i)
			len += sprintf (tmp + len, "%.6e\t ", res [i].obj_val);
	}
	len += sprintf (tmp + len, "%s %c\n", statusIDs, gpu_fail);

	xprint (3) ("%s", tmp);

	FILE* f = fopen (logname, "a+");
	fwrite (tmp, len, 1, f);
	fclose (f);

	if (ABORT_ON_DEVIATE && gpu_fail == '!')
		exit (0);
}
