/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	June 06, 2009


#pragma once

#include <limits>
#include <string>
#include <vector>
#include <map>



template <typename S>
inline bool isinf (S s)
{
	return s == std::numeric_limits <S>::infinity () ||
		  -s == std::numeric_limits <S>::infinity ();
}

enum bounds_e
{
	no_bounds = 0,
	lower_bound = 1,	//	<= 2 means has lower
	upper_bound = 3,	//	>= 2 means has upper
	double_bound = 2
};

struct lp_result_t;
struct solver_t;

struct lp_problem_t
{
	bool save_txt (char const* fname);
	bool load_txt (char const* file);
	bool load_mps (char const* fname);

	void set_random (unsigned seed, int rows, int cols, bool negative_b);
	void set_random_bnd (unsigned seed, int rows, int cols, bool negative_b);
	void set_random_old (unsigned seed, int rows, int cols, bool negative_b);

	bool solve (solver_t const& solver, char const* prefix, lp_result_t& res) const;

	std::vector <double> const& operator [] (std::string const& str) const
	{
		return data.find (str)->second;
	}
	
	std::vector <double>& operator [] (std::string const& str)
	{
		return data.find (str)->second;
	}
	
	bool has (char const* mat_name) const
	{
		return data.find (mat_name) != data.end ();
	}

private:
	std::map < std::string, std::vector <double> >	data;
};

