/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	February 16, 2010


#pragma once

#ifdef _WIN32

#include <cstdio>
#include <conio.h>	//	_kbhit
#include <windows.h>

#ifdef min
#undef min
#endif

#ifdef max
#undef max
#endif

#define remove DeleteFile

inline void beep ()
{
	MessageBeep (MB_ICONEXCLAMATION);
}

inline FILE* fopentmp (char const*& name)
{
	static char tmp [MAX_PATH * 2];
	unsigned sz = GetTempPath (MAX_PATH, &tmp [0]);
	GetTempFileName (".\\", "mps", 0, &tmp [sz]);
	
	name = tmp;
	return fopen (name, "w");
}

inline bool set_realtime_priority ()
{
    return SetPriorityClass (GetCurrentProcess(), HIGH_PRIORITY_CLASS) != 0;
}

#else

#include <errno.h>
#include <curses.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <cstring>

#define stricmp strcasecmp
#define strnicmp strncasecmp

inline void Sleep (unsigned long msec)
{
	usleep (msec * 1000);
}

inline FILE* fopentmp (char const*& name)
{
	static char tmp [L_tmpnam * 2];
	name = tmpnam (tmp);
	return fopen (name, "w");
}

inline bool set_realtime_priority ()
{
	errno = 0;
	setpriority (PRIO_PROCESS, 0, -20);
	return errno == 0;
}

inline const char* itoa (int i, char* buff, int radix)
{
	static char tmp [32];
	
	int cur = sizeof (tmp) - 1;
	tmp [cur] = 0;
	while (i > 0)
	{
		-- cur;
		tmp [cur] = (i % radix) + '0';
		i /= radix;
	}
	
	strcpy (buff, tmp + cur);
	return buff;
}

inline int _kbhit ()
{
	timeval tv;
	fd_set rdfs;
	
	tv.tv_sec = 0;
	tv.tv_usec = 0;
	
	FD_ZERO (&rdfs);
	FD_SET (STDIN_FILENO, &rdfs);
	
	select (STDIN_FILENO + 1, &rdfs, NULL, NULL, &tv);
	return FD_ISSET (STDIN_FILENO, &rdfs);

}

#endif
