/**	gpu_simplex, a simplex solver using the GPU
 *	Copyright (C) 2011  Jakob Bieling, University of Bonn <bieling@cs.uni-bonn.de>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//	Author:	Jakob Bieling
//	Date:	May 20, 2010


#pragma once

#include "arg_parser.h"


//	these defines are set in the Makefile for Linux/MacOSX
#ifdef WIN32
	#define USE_GPU
	#define USE_GLPK
#endif


enum implementations_e
{
	imp_gpu = 0,
	imp_glpk = 1,
};

struct lp_problem_t;
struct lp_result_t;

struct solver_t
{
	bool		skip;
	char const*	name;
	void (*solve) (lp_problem_t const&, lp_result_t&);
};



void solve_gpu (lp_problem_t const& prob, lp_result_t& res);
void solve_glpk (lp_problem_t const& prob, lp_result_t& res);


inline void not_implemented (char const* msg)
{
	xprint (0) ("Not implemented: %s\n", msg);
}
