/**	make_h, convert text files into include files
 *	Copyright (C) 2011  Jakob Bieling
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <iomanip>
#include <fstream>
#include <iterator>
#include <vector>
#include <algorithm>
#include <cstring>

#define VERSION	"1.0"

void print_help ()
{
	std::cout << "make_h  Copyright (C) 2011  Jakob Bieling <bieling at cs.uni-bonn.de>\n"
				 "This program is free software: you can redistribute it and/or modify\n"
				 "it under the terms of the GNU General Public License as published by\n"
				 "the Free Software Foundation, either version 3 of the License, or\n"
				 "(at your option) any later version.\n"
				 "\n"
				 "This program is distributed in the hope that it will be useful,\n"
				 "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
				 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
				 "GNU General Public License for more details.\n"
				 "\n"
				 "You should have received a copy of the GNU General Public License\n"
				 "along with this program.  If not, see <http://www.gnu.org/licenses/>.\n" << std::endl;
	std::cout << "Usage: make_h [--text | --bin] FILE" << std::endl;
}

bool invalid_char (char c)
{
	return !isalnum ((unsigned char) c) && c != '_';
}

void replace_all (std::string& str, char const* find, char const* repl)
{
	size_t find_len = strlen (find);
	size_t repl_len = strlen (repl);
	
	size_t pos = 0;
	while ((pos = str.find (find, pos)) != std::string::npos)
	{
		str.replace (pos, find_len, repl, repl_len);
		pos += repl_len;
	}
}

int main (int argc, char const* argv[])
{
	bool		text = true;
	char const*	file = 0;
	
	if (argc < 2)
	{
		print_help ();
		return 0;
	}

	if (argc > 2)
	{
		if (strcasecmp (argv [1], "--bin"))
			text = false;
		else if (strcasecmp (argv [1], "--text"))
			text = false;
		else
		{
			print_help ();
			std::cout << "Error: invalid argument" << std::endl;
			return 0;
		}
		
		file = argv [2];
	}
	else
		file = argv [1];

	std::ifstream f (file, text ? std::ios_base::in :  std::ios_base::in | std::ios_base::binary);
	if (!f)
	{
		print_help ();
		std::cout << "Error: unable to open file" << std::endl;
		return 0;
	}

	std::string nm;
	if (strrchr (file, '/'))
		nm.assign (strrchr (file, '/') + 1);
	else
		nm.assign (file);
	std::replace_if (nm.begin (), nm.end (), invalid_char, '_');

	std::cout << "// File: " << file << std::endl;
	if (text)
	{
		std::cout << "char const* const " << nm << " = {" << std::endl;

		std::string ln;
		while (getline (f, ln))
		{
			if (*ln.rbegin () == '\r')
				ln.erase (ln.size () - 1);
			replace_all (ln, "\\", "\\\\");
			replace_all (ln, "\"", "\\\"");
			std::cout << "\t\"" << ln << "\\n\"" << std::endl;
		}

		std::cout << "};" << std::endl;
	}
	else
	{
		std::cout << "unsigned char const* const " << nm << " = {";

		std::vector <char> buff;
		std::copy (std::istream_iterator <char> (f), std::istream_iterator <char> (), std::back_inserter (buff));

		size_t c = 0;
		while (c != buff.size () - 1)
		{
			size_t ce = std::min (c + 20, buff.size () - 1);

			std::cout << std::endl << "\t";
			for (; c != ce; ++ c)
				std::cout << "0x" << std::hex << std::setw (2) << static_cast <unsigned> (buff [c]) << ", ";
		}

		std::cout << "0x" << std::hex << std::setw (2) << static_cast <unsigned> (buff [c]) << std::endl << "};" << std::endl;
	}
	
	f.close ();
    return 0;
}
